###############################################################################

module TerraformWrapper

###############################################################################

  module Shared

###############################################################################

    module Backends

###############################################################################

      class Local < Common

###############################################################################

        include TerraformWrapper::Shared::Logging

###############################################################################

        @@type = "local"

###############################################################################

        attr_reader :path

###############################################################################

        def initialize(options:, variables:)
          construct(options: options, variables: variables)
        end

###############################################################################

        def hash()
          return {
            "path" => @path
          }
        end

###############################################################################

        private

###############################################################################

        def specific()
          path = @options.key?("path") ? @options["path"] : File.join(Dir.pwd, "state", "terraform", "%{config}", "%{component}" + @@ext)

          logger.fatal("Local backend path must be a string!") unless path.kind_of?(String)
          logger.fatal("Local backend path must not be blank!") if path.strip.empty?

          @variables.core.keys.map{ |sym| sym.to_s }.each do |core|
            next if (core == "service") and (path.include?(Dir.pwd))
            logger.fatal("Local backend path must include %{#{core}}.") unless path.include?("%{#{core}}")
          end

          begin
            path = path % @variables.identifiers
          rescue
            logger.fatal("Local backend options contain identifiers that are not included in the configuration file!")
          end

          directory = File.dirname(path)

          logger.fatal("Failed to create state directory: #{directory}") unless ::TerraformWrapper.create_directory(directory: directory, purpose: "state")

          @path = path
        end

###############################################################################

      end

###############################################################################

    end

###############################################################################

  end

###############################################################################

end

###############################################################################
