###############################################################################

module TerraformWrapper

###############################################################################

  module Tasks

###############################################################################

    class Apply < ::Rake::TaskLib

###############################################################################

      include TerraformWrapper::Shared::Logging

###############################################################################

      @binary
      @code
      @options

###############################################################################

      def initialize(binary:, code:, options:)
        @binary  = binary
        @code    = code
        @options = options

        yield self if block_given?

        apply_task
      end

###############################################################################

      def apply_task
        desc "Applies infrastructure with Terraform for a given configuration on an infrastructure component."
        task :apply, [:config, :plan] => :binary do |t, args|
          options = @options.merge({"name" => args[:config]})

          logger.info("Processing configuration for Terraform apply...")

          config = TerraformWrapper::Shared::Config.new(code: @code, options: options)
          runner = TerraformWrapper::Shared::Runner.new(binary: @binary, code: @code)

          logger.info("Running Terraform apply for service: #{config.service}, component: #{@code.name}...")

          runner.init(config: config)
          runner.apply(file: args[:plan])
        end
      end

###############################################################################

    end

###############################################################################

  end

###############################################################################

end

###############################################################################
