###############################################################################

module TerraformWrapper

###############################################################################

  module Tasks

###############################################################################

    class Plan < ::Rake::TaskLib

###############################################################################

      include TerraformWrapper::Shared::Logging

###############################################################################

      @binary
      @code
      @options

###############################################################################

      def initialize(binary:, code:, options:)
        @binary  = binary
        @code    = code
        @options = options

        yield self if block_given?

        plan_task
      end

###############################################################################

      def plan_task
        desc "Creates a Terraform plan for a given configuration on an infrastructure component."
        task :plan, [:config, :out] => :binary do |t, args|
          options = @options.merge({"name" => args[:config]})

          logger.info("Processing configuration for Terraform plan...")

          config = TerraformWrapper::Shared::Config.new(code: @code, options: options)
          runner = TerraformWrapper::Shared::Runner.new(binary: @binary, code: @code)

          logger.info("Running Terraform plan for service: #{config.service}, component: #{@code.name}...")

          runner.init(config: config)
          runner.plan(file: args[:out])
        end
      end

###############################################################################

    end

###############################################################################

  end

###############################################################################

end

###############################################################################
